@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Lexer;
 
-our $VERSION = 'v0.39.0'; # VERSION
+our $VERSION = '0.021'; # VERSION
 
 use constant TRACE => $ENV{YAML_PP_TRACE} ? 1 : 0;
 use constant DEBUG => ($ENV{YAML_PP_DEBUG} || $ENV{YAML_PP_TRACE}) ? 1 : 0;
@@ -42,8 +42,6 @@ sub context { return $_[0]->{context} }
 sub set_context { $_[0]->{context} = $_[1] }
 sub flowcontext { return $_[0]->{flowcontext} }
 sub set_flowcontext { $_[0]->{flowcontext} = $_[1] }
-sub block { return $_[0]->{block} }
-sub set_block { $_[0]->{block} = $_[1] }
 
 my $RE_WS = '[\t ]';
 my $RE_LB = '[\r\n]';
@@ -55,7 +53,7 @@ my $RE_EOL = qr/\A($RE_WS+#.*|$RE_WS+)\z/;
 #ns-word-char    ::= ns-dec-digit | ns-ascii-letter | “-”
 my $RE_NS_WORD_CHAR = '[0-9A-Za-z-]';
 my $RE_URI_CHAR = '(?:' . '%[0-9a-fA-F]{2}' .'|'.  q{[0-9A-Za-z#;/?:@&=+$,_.!*'\(\)\[\]-]} . ')';
-my $RE_NS_TAG_CHAR = '(?:' . '%[0-9a-fA-F]{2}' .'|'.  q{[0-9A-Za-z#;/?:@&=+$_.~*'\(\)-]} . ')';
+my $RE_NS_TAG_CHAR = '(?:' . '%[0-9a-fA-F]{2}' .'|'.  q{[0-9A-Za-z#;/?:@&=+$_.*'\(\)-]} . ')';
 
 #  [#x21-#x7E]          /* 8 bit */
 # | #x85 | [#xA0-#xD7FF] | [#xE000-#xFFFD] /* 16 bit */
@@ -131,7 +129,7 @@ my %REGEXES = (
     SINGLEQUOTED => qr{(?:''|[^'\r\n]+)*},
 );
 
-sub _fetch_next_line {
+sub fetch_next_line {
     my ($self) = @_;
     my $next_line = $self->next_line;
     if (defined $next_line ) {
@@ -143,7 +141,6 @@ sub _fetch_next_line {
         $self->set_next_line(undef);
         return;
     }
-    $self->set_block(1);
     $self->inc_line;
     $line =~ m/\A( *)([^\r\n]*)([\r\n]|\z)/ or die "Unexpected";
     $next_line = [ $1,  $2, $3 ];
@@ -182,7 +179,7 @@ sub fetch_next_tokens {
     my $next = $self->next_tokens;
     return $next if @$next;
 
-    my $next_line = $self->_fetch_next_line;
+    my $next_line = $self->fetch_next_line;
     if (not $next_line) {
         return [];
     }
@@ -190,12 +187,12 @@ sub fetch_next_tokens {
     my $spaces = $next_line->[0];
     my $yaml = \$next_line->[1];
     if (not length $$yaml) {
-        $self->_push_tokens([ EOL => join('', @$next_line), $self->line ]);
+        $self->push_tokens([ EOL => join('', @$next_line), $self->line ]);
         $self->set_next_line(undef);
         return $next;
     }
     if (substr($$yaml, 0, 1) eq '#') {
-        $self->_push_tokens([ EOL => join('', @$next_line), $self->line ]);
+        $self->push_tokens([ EOL => join('', @$next_line), $self->line ]);
         $self->set_next_line(undef);
         return $next;
     }
@@ -206,15 +203,10 @@ sub fetch_next_tokens {
         return $next;
     }
     if (not $spaces and $$yaml =~ s/\A(---|\.\.\.)(?=$RE_WS|\z)//) {
-        $self->_push_tokens([ $TOKEN_NAMES{ $1 } => $1, $self->line ]);
-    }
-    elsif ($self->flowcontext and $$yaml =~ m/\A[ \t]+(#.*)?\z/) {
-        $self->_push_tokens([ EOL => join('', @$next_line), $self->line ]);
-        $self->set_next_line(undef);
-        return $next;
+        $self->push_tokens([ $TOKEN_NAMES{ $1 } => $1, $self->line ]);
     }
     else {
-        $self->_push_tokens([ SPACE => $spaces, $self->line ]);
+        $self->push_tokens([ SPACE => $spaces, $self->line ]);
     }
 
     my $partial = $self->_fetch_next_tokens($next_line);
@@ -232,12 +224,12 @@ my %FLOW =                ( '{' => 1, '[' => 1, '}' => 1, ']' => 1, ',' => 1 );
 my %CONTEXT =             ( '"' => 1, "'" => 1, '>' => 1, '|' => 1 );
 
 my $RE_ESCAPES = qr{(?:
-    \\([ \\\/_0abefnrtvLNP\t"]) | \\x([0-9a-fA-F]{2})
+    \\([ \\\/_0abefnrtvLNP"]) | \\x([0-9a-fA-F]{2})
     | \\u([A-Fa-f0-9]{4}) | \\U([A-Fa-f0-9]{4,8})
 )}x;
 my %CONTROL = (
     '\\' => '\\', '/' => '/', n => "\n", t => "\t", r => "\r", b => "\b",
-    'a' => "\a", 'b' => "\b", 'e' => "\e", 'f' => "\f", 'v' => "\x0b", "\t" => "\t",
+    'a' => "\a", 'b' => "\b", 'e' => "\e", 'f' => "\f", 'v' => "\x0b",
     'P' => "\x{2029}", L => "\x{2028}", 'N' => "\x85",
     '0' => "\0", '_' => "\xa0", ' ' => ' ', q/"/ => q/"/,
 );
@@ -254,7 +246,7 @@ sub _fetch_next_tokens {
     while (1) {
         unless (length $$yaml) {
             push @tokens, ( EOL => $eol, $self->line );
-            $self->_push_tokens(\@tokens);
+            $self->push_tokens(\@tokens);
             return;
         }
         my $first = substr($$yaml, 0, 1);
@@ -269,39 +261,30 @@ sub _fetch_next_tokens {
             }
             if ($$yaml =~ s/\A($RE_WS*(?: #.*))\z//) {
                 push @tokens, ( EOL => $1 . $eol, $self->line );
-                $self->_push_tokens(\@tokens);
+                $self->push_tokens(\@tokens);
                 return;
             }
             $self->set_context(0);
         }
         if ($CONTEXT{ $first }) {
             push @tokens, ( CONTEXT => $first, $self->line );
-            $self->_push_tokens(\@tokens);
+            $self->push_tokens(\@tokens);
             return 1;
         }
         elsif ($COLON_DASH_QUESTION{ $first }) {
             my $token_name = $TOKEN_NAMES{ $first };
-            if ($$yaml =~ s/\A\Q$first\E($RE_WS+|\z)//) {
-                my $after = $1;
-                if (not $self->flowcontext and not $self->block) {
-                    push @tokens, ERROR => $first . $after, $self->line;
-                    $self->_push_tokens(\@tokens);
-                    $self->exception("Tabs can not be used for indentation");
-                }
-                if ($after =~ tr/\t//) {
-                    $self->set_block(0);
-                }
+            if ($$yaml =~ s/\A\Q$first\E(?:($RE_WS+)|\z)//) {
                 my $token_name = $TOKEN_NAMES{ $first };
                 push @tokens, ( $token_name => $first, $self->line );
                 if (not defined $1) {
                     push @tokens, ( EOL => $eol, $self->line );
-                    $self->_push_tokens(\@tokens);
+                    $self->push_tokens(\@tokens);
                     return;
                 }
                 my $ws = $1;
                 if ($$yaml =~ s/\A(#.*|)\z//) {
                     push @tokens, ( EOL => $ws . $1 . $eol, $self->line );
-                    $self->_push_tokens(\@tokens);
+                    $self->push_tokens(\@tokens);
                     return;
                 }
                 push @tokens, ( WS => $ws, $self->line );
@@ -321,7 +304,7 @@ sub _fetch_next_tokens {
             }
             else {
                 push @tokens, ( "Invalid $token_name" => $$yaml, $self->line );
-                $self->_push_tokens(\@tokens);
+                $self->push_tokens(\@tokens);
                 return;
             }
         }
@@ -330,7 +313,7 @@ sub _fetch_next_tokens {
                 my $ws = $1;
                 if ($$yaml =~ s/\A((?:#.*)?\z)//) {
                     push @tokens, ( EOL => $ws . $1 . $eol, $self->line );
-                    $self->_push_tokens(\@tokens);
+                    $self->push_tokens(\@tokens);
                     return;
                 }
                 push @tokens, ( WS => $ws, $self->line );
@@ -353,7 +336,7 @@ sub _fetch_next_tokens {
 
         if ($plain) {
             push @tokens, ( CONTEXT => '', $self->line );
-            $self->_push_tokens(\@tokens);
+            $self->push_tokens(\@tokens);
             return 1;
         }
 
@@ -373,8 +356,8 @@ sub fetch_plain {
     }
 
     my @tokens;
-    unless ($$yaml =~ s/\A($REGEX(?:[:]+(?=\:(\s|\z)))?)//) {
-        $self->_push_tokens(\@tokens);
+    unless ($$yaml =~ s/\A($REGEX)//) {
+        $self->push_tokens(\@tokens);
         $self->exception("Invalid plain scalar");
     }
     my $plain = $1;
@@ -383,7 +366,7 @@ sub fetch_plain {
     if ($$yaml =~ s/\A(?:($RE_WS+#.*)|($RE_WS*))\z//) {
         if (defined $1) {
             push @tokens, ( EOL => $1 . $eol, $self->line );
-            $self->_push_tokens(\@tokens);
+            $self->push_tokens(\@tokens);
             $self->set_next_line(undef);
             return;
         }
@@ -393,7 +376,7 @@ sub fetch_plain {
         }
     }
     else {
-        $self->_push_tokens(\@tokens);
+        $self->push_tokens(\@tokens);
         my $partial = $self->_fetch_next_tokens($next_line);
         if (not $partial) {
             $self->set_next_line(undef);
@@ -409,7 +392,7 @@ sub fetch_plain {
     my @lines = ($plain);
     my @next;
     LOOP: while (1) {
-        $next_line = $self->_fetch_next_line;
+        $next_line = $self->fetch_next_line;
         if (not $next_line) {
             last LOOP;
         }
@@ -499,10 +482,10 @@ sub fetch_plain {
             @eol = splice @tokens, -3;
         }
         $self->push_subtokens( { name => 'PLAIN_MULTI', value => $value }, \@tokens);
-        $self->_push_tokens([ @eol, @next ]);
+        $self->push_tokens([ @eol, @next ]);
     }
     else {
-        $self->_push_tokens([ @tokens, @next ]);
+        $self->push_tokens([ @tokens, @next ]);
     }
     @tokens = ();
     if ($fetch_next) {
@@ -528,13 +511,13 @@ sub fetch_block {
     my $started = 0;
     my $set_indent = 0;
     my $chomp = '';
-    if ($$yaml =~ s/\A([1-9])([+-]?)//) {
+    if ($$yaml =~ s/\A([1-9]\d*)([+-]?)//) {
         push @tokens, ( BLOCK_SCALAR_INDENT => $1, $self->line );
         $set_indent = $1;
         $chomp = $2 if $2;
         push @tokens, ( BLOCK_SCALAR_CHOMP => $2, $self->line ) if $2;
     }
-    elsif ($$yaml =~ s/\A([+-])([1-9])?//) {
+    elsif ($$yaml =~ s/\A([+-])([1-9]\d*)?//) {
         push @tokens, ( BLOCK_SCALAR_CHOMP => $1, $self->line );
         $chomp = $1;
         push @tokens, ( BLOCK_SCALAR_INDENT => $2, $self->line ) if $2;
@@ -542,8 +525,7 @@ sub fetch_block {
     }
     if ($set_indent) {
         $started = 1;
-        $indent-- if $indent > 0;
-        $current_indent = $indent + $set_indent;
+        $current_indent = $set_indent;
     }
     if (not length $$yaml) {
         push @tokens, ( EOL => $eol, $self->line );
@@ -552,14 +534,14 @@ sub fetch_block {
         push @tokens, ( EOL => $1 . $eol, $self->line );
     }
     else {
-        $self->_push_tokens(\@tokens);
+        $self->push_tokens(\@tokens);
         $self->exception("Invalid block scalar");
     }
 
     my @lines;
     while (1) {
         $self->set_next_line(undef);
-        $next_line = $self->_fetch_next_line;
+        $next_line = $self->fetch_next_line;
         if (not $next_line) {
             last;
         }
@@ -571,10 +553,6 @@ sub fetch_block {
         }
         if ((length $spaces) < $current_indent) {
             if (length $content) {
-                if ($content =~ m/\A\t/) {
-                    $self->_push_tokens(\@tokens);
-                    $self->exception("Invalid block scalar");
-                }
                 last;
             }
             else {
@@ -611,7 +589,7 @@ sub fetch_block {
     my $value = YAML::PP::Render->render_block_scalar($context, $chomp, \@lines);
     my @eol = splice @tokens, -3;
     $self->push_subtokens( { name => 'BLOCK_SCALAR', value => $value }, \@tokens );
-    $self->_push_tokens([ @eol ]);
+    $self->push_tokens([ @eol ]);
     return 0;
 }
 
@@ -630,14 +608,14 @@ sub fetch_quoted {
     while (1) {
 
         unless ($start) {
-            $next_line = $self->_fetch_next_line or do {
+            $next_line = $self->fetch_next_line or do {
                     for (my $i = 0; $i < @tokens; $i+= 3) {
                         my $token = $tokens[ $i + 1 ];
                         if (ref $token) {
                             $tokens[ $i + 1 ] = $token->{orig};
                         }
                     }
-                    $self->_push_tokens(\@tokens);
+                    $self->push_tokens(\@tokens);
                     $self->exception("Missing closing quote <$context> at EOF");
                 };
             $start = 0;
@@ -657,7 +635,7 @@ sub fetch_quoted {
                             $tokens[ $i + 1 ] = $token->{orig};
                         }
                     }
-                $self->_push_tokens(\@tokens);
+                $self->push_tokens(\@tokens);
                 $self->exception("Missing closing quote <$context> or invalid document marker");
             }
             elsif ((length $spaces) < $indent) {
@@ -667,7 +645,7 @@ sub fetch_quoted {
                         $tokens[ $i + 1 ] = $token->{orig};
                     }
                 }
-                $self->_push_tokens(\@tokens);
+                $self->push_tokens(\@tokens);
                 $self->exception("Wrong indendation or missing closing quote <$context>");
             }
 
@@ -702,7 +680,7 @@ sub fetch_quoted {
             else {
                 @tokens = ();
                 push @tokens, ( EOL => $next_line->[2], $self->line );
-                $self->_push_tokens(\@tokens);
+                $self->push_tokens(\@tokens);
                 $self->set_next_line(undef);
                 return;
             }
@@ -718,7 +696,6 @@ sub _read_quoted_tokens {
     my $quoted = '';
     my $decoded = '';
     my $token_name = $TOKEN_NAMES{ $first };
-    my $eol = '';
     if ($first eq "'") {
         my $regex = $REGEXES{SINGLEQUOTED};
         if ($$yaml =~ s/\A($regex)//) {
@@ -726,15 +703,16 @@ sub _read_quoted_tokens {
             $decoded .= $1;
             $decoded =~ s/''/'/g;
         }
-        unless (length $$yaml) {
-            if ($quoted =~ s/($RE_WS+)\z//) {
-                $eol = $1;
-                $decoded =~ s/($eol)\z//;
-            }
-        }
     }
     else {
-        ($quoted, $decoded, $eol) = $self->_read_doublequoted($yaml);
+        ($quoted, $decoded) = $self->_read_doublequoted($yaml);
+    }
+    my $eol = '';
+    unless (length $$yaml) {
+        if ($quoted =~ s/($RE_WS+)\z//) {
+            $eol = $1;
+            $decoded =~ s/($eol)\z//;
+        }
     }
     my $value = { value => $decoded, orig => $quoted };
 
@@ -750,7 +728,7 @@ sub _read_quoted_tokens {
     }
     if (length $$yaml) {
         push @$tokens, ( $token_name . 'D' => $value->{orig}, $self->line );
-        $self->_push_tokens($tokens);
+        $self->push_tokens($tokens);
         $self->exception("Invalid quoted <$first> string");
     }
 
@@ -764,10 +742,9 @@ sub _read_doublequoted {
     my ($self, $yaml) = @_;
     my $quoted = '';
     my $decoded = '';
-    my $eol = '';
     while (1) {
         my $last = 1;
-        if ($$yaml =~ s/\A([^"\\ \t]+)//) {
+        if ($$yaml =~ s/\A([^"\\]+)//) {
             $quoted .= $1;
             $decoded .= $1;
             $last = 0;
@@ -781,18 +758,6 @@ sub _read_doublequoted {
             $decoded .= $dec;
             $last = 0;
         }
-        if ($$yaml =~ s/\A([ \t]+)//) {
-            my $spaces = $1;
-            if (length $$yaml) {
-                $quoted .= $spaces;
-                $decoded .= $spaces;
-                $last = 0;
-            }
-            else {
-                $eol = $spaces;
-                last;
-            }
-        }
         if ($$yaml =~ s/\A(\\)\z//) {
             $quoted .= $1;
             $decoded .= $1;
@@ -800,47 +765,52 @@ sub _read_doublequoted {
         }
         last if $last;
     }
-    return ($quoted, $decoded, $eol);
+    return ($quoted, $decoded);
 }
 
 sub _fetch_next_tokens_directive {
     my ($self, $yaml, $eol) = @_;
     my @tokens;
 
-    my $trailing_ws = '';
-    my $warn = $ENV{YAML_PP_RESERVED_DIRECTIVE} || 'warn';
-    if ($$yaml =~ s/\A(\s*%YAML[ \t]+([0-9]+\.[0-9]+))//) {
+    if ($$yaml =~ s/\A(\s*%YAML)//) {
         my $dir = $1;
-        my $version = $2;
-        if ($$yaml =~ s/\A($RE_WS+)//) {
-            $trailing_ws = $1;
+        if ($$yaml =~ s/\A( )//) {
+            $dir .= $1;
+            if ($$yaml =~ s/\A(1\.[12]$RE_WS*)//) {
+                $dir .= $1;
+                push @tokens, ( YAML_DIRECTIVE => $dir, $self->line );
+            }
+            else {
+                $$yaml =~ s/\A(.*)//;
+                $dir .= $1;
+                my $warn = $ENV{YAML_PP_RESERVED_DIRECTIVE} || 'warn';
+                if ($warn eq 'warn') {
+                    warn "Found reserved directive '$dir'";
+                }
+                elsif ($warn eq 'fatal') {
+                    die "Found reserved directive '$dir'";
+                }
+                push @tokens, ( RESERVED_DIRECTIVE => "$dir", $self->line );
+            }
         }
-        elsif (length $$yaml) {
-            push @tokens, ( 'Invalid directive' => $dir.$$yaml.$eol, $self->line );
-            $self->_push_tokens(\@tokens);
+        else {
+            $$yaml =~ s/\A(.*)//;
+            $dir .= $1;
+            push @tokens, ( 'Invalid directive' => $dir, $self->line );
+            push @tokens, ( EOL => $eol, $self->line );
+            $self->push_tokens(\@tokens);
             return;
         }
-        if ($version !~ m/^1\.[12]$/) {
-            if ($warn eq 'warn') {
-                warn "Unsupported YAML version '$dir'";
-            }
-            elsif ($warn eq 'fatal') {
-                push @tokens, ( 'Unsupported YAML version' => $dir, $self->line );
-                $self->_push_tokens(\@tokens);
-                return;
-            }
-        }
-        push @tokens, ( YAML_DIRECTIVE => $dir, $self->line );
     }
-    elsif ($$yaml =~ s/\A(\s*%TAG[ \t]+(!$RE_NS_WORD_CHAR*!|!)[ \t]+(tag:\S+|!$RE_URI_CHAR+))($RE_WS*)//) {
+    elsif ($$yaml =~ s/\A(\s*%TAG +(!$RE_NS_WORD_CHAR*!|!) +(tag:\S+|!$RE_URI_CHAR+)$RE_WS*)//) {
         push @tokens, ( TAG_DIRECTIVE => $1, $self->line );
         # TODO
         my $tag_alias = $2;
         my $tag_url = $3;
-        $trailing_ws = $4;
     }
     elsif ($$yaml =~ s/\A(\s*\A%(?:\w+).*)//) {
         push @tokens, ( RESERVED_DIRECTIVE => $1, $self->line );
+        my $warn = $ENV{YAML_PP_RESERVED_DIRECTIVE} || 'warn';
         if ($warn eq 'warn') {
             warn "Found reserved directive '$1'";
         }
@@ -851,31 +821,21 @@ sub _fetch_next_tokens_directive {
     else {
         push @tokens, ( 'Invalid directive' => $$yaml, $self->line );
         push @tokens, ( EOL => $eol, $self->line );
-        $self->_push_tokens(\@tokens);
+        $self->push_tokens(\@tokens);
         return;
     }
     if (not length $$yaml) {
         push @tokens, ( EOL => $eol, $self->line );
     }
-    elsif ($trailing_ws and $$yaml =~ s/\A(#.*)?\z//) {
-        push @tokens, ( EOL => "$trailing_ws$1$eol", $self->line );
-        $self->_push_tokens(\@tokens);
-        return;
-    }
-    elsif ($$yaml =~ s/\A([ \t]+#.*)?\z//) {
-        push @tokens, ( EOL => "$1$eol", $self->line );
-        $self->_push_tokens(\@tokens);
-        return;
-    }
     else {
-        push @tokens, ( 'Invalid directive' => $trailing_ws.$$yaml, $self->line );
+        push @tokens, ( 'Invalid directive' => $$yaml, $self->line );
         push @tokens, ( EOL => $eol, $self->line );
     }
-    $self->_push_tokens(\@tokens);
+    $self->push_tokens(\@tokens);
     return;
 }
 
-sub _push_tokens {
+sub push_tokens {
     my ($self, $new_tokens) = @_;
     my $next = $self->next_tokens;
     my $line = $self->line;
