@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Representer;
 
-our $VERSION = 'v0.38.0'; # VERSION
+our $VERSION = '0.021'; # VERSION
 
 use Scalar::Util qw/ reftype blessed refaddr /;
 
@@ -13,15 +13,15 @@ use YAML::PP::Common qw/
     YAML_LITERAL_SCALAR_STYLE YAML_FOLDED_SCALAR_STYLE
     YAML_FLOW_SEQUENCE_STYLE YAML_FLOW_MAPPING_STYLE
     YAML_BLOCK_MAPPING_STYLE YAML_BLOCK_SEQUENCE_STYLE
-    PRESERVE_ORDER PRESERVE_SCALAR_STYLE PRESERVE_FLOW_STYLE PRESERVE_ALIAS
+    PRESERVE_ALL PRESERVE_ORDER PRESERVE_SCALAR_STYLE
 /;
 use B;
 
 sub new {
     my ($class, %args) = @_;
     my $preserve = delete $args{preserve} || 0;
-    if ($preserve == 1) {
-        $preserve = PRESERVE_ORDER | PRESERVE_SCALAR_STYLE | PRESERVE_FLOW_STYLE | PRESERVE_ALIAS;
+    if ($preserve == PRESERVE_ALL) {
+        $preserve = PRESERVE_ORDER | PRESERVE_SCALAR_STYLE;
     }
     my $self = bless {
         schema => delete $args{schema},
@@ -45,18 +45,14 @@ sub clone {
 sub schema { return $_[0]->{schema} }
 sub preserve_order { return $_[0]->{preserve} & PRESERVE_ORDER }
 sub preserve_scalar_style { return $_[0]->{preserve} & PRESERVE_SCALAR_STYLE }
-sub preserve_flow_style { return $_[0]->{preserve} & PRESERVE_FLOW_STYLE }
-sub preserve_alias { return $_[0]->{preserve} & PRESERVE_ALIAS }
 
 sub represent_node {
     my ($self, $node) = @_;
 
-    my $preserve_alias = $self->preserve_alias;
-    my $preserve_style = $self->preserve_scalar_style;
-    if ($preserve_style or $preserve_alias) {
+    if ($self->preserve_scalar_style) {
         if (ref $node->{value} eq 'YAML::PP::Preserve::Scalar') {
             my $value = $node->{value}->value;
-            if ($preserve_style) {
+            if ($node->{value}->style != YAML_FOLDED_SCALAR_STYLE) {
                 $node->{style} = $node->{value}->style;
             }
 #            $node->{tag} = $node->{value}->tag;
@@ -64,15 +60,12 @@ sub represent_node {
         }
     }
     $node->{reftype} = reftype($node->{value});
-    if (not $node->{reftype} and reftype(\$node->{value}) eq 'GLOB') {
-        $node->{reftype} = 'GLOB';
-    }
 
-    if ($node->{reftype}) {
-        $self->_represent_noderef($node);
+    if (ref $node->{value}) {
+        $self->represent_noderef($node);
     }
     else {
-        $self->_represent_node_nonref($node);
+        $self->represent_node_nonref($node);
     }
     $node->{reftype} = (reftype $node->{data}) || '';
 
@@ -99,28 +92,16 @@ sub represent_node {
                 push @{ $node->{items} }, $key, $node->{data}->{ $key };
             }
         }
-        my %args;
-        if ($self->preserve_flow_style and reftype $node->{value} eq 'HASH') {
-            if (my $tied = tied %{ $node->{value} } ) {
-                $args{style} = $tied->{style};
-            }
-        }
-        return [ mapping => $node, %args ];
+        return [ mapping => $node ];
     }
     elsif ($node->{reftype} eq 'ARRAY') {
         unless (defined $node->{items}) {
             @{ $node->{items} } = @{ $node->{data} };
         }
-        my %args;
-        if ($self->preserve_flow_style and reftype $node->{value} eq 'ARRAY') {
-            if (my $tied = tied @{ $node->{value} } ) {
-                $args{style} = $tied->{style};
-            }
-        }
-        return [ sequence => $node, %args ];
+        return [ sequence => $node ];
     }
     elsif ($node->{reftype}) {
-        die "Cannot handle reftype '$node->{reftype}' (you might want to enable YAML::PP::Schema::Perl)";
+        die "Reftype $node->{reftype} not implemented";
     }
     else {
         unless (defined $node->{items}) {
@@ -131,16 +112,7 @@ sub represent_node {
 
 }
 
-my $bool_code = <<'EOM';
-sub {
-    my ($x) = @_;
-    use experimental qw/ builtin /;
-    builtin::is_bool($x);
-}
-EOM
-my $is_bool;
-
-sub _represent_node_nonref {
+sub represent_node_nonref {
     my ($self, $node) = @_;
     my $representers = $self->schema->representers;
 
@@ -154,12 +126,6 @@ sub _represent_node_nonref {
             return 1;
         }
     }
-    if ($] >= 5.036000 and my $rep = $representers->{bool}) {
-        $is_bool ||= eval $bool_code;
-        if ($is_bool->($node->{value})) {
-            return $rep->{code}->($self, $node);
-        }
-    }
     for my $rep (@{ $representers->{flags} }) {
         my $check_flags = $rep->{flags};
         my $flags = B::svref_2object(\$node->{value})->FLAGS;
@@ -185,7 +151,7 @@ sub _represent_node_nonref {
     }
 }
 
-sub _represent_noderef {
+sub represent_noderef {
     my ($self, $node) = @_;
     my $representers = $self->schema->representers;
 
@@ -219,10 +185,6 @@ sub _represent_noderef {
         my $code = $coderef->{code};
         return 1 if $code->($self, $node);
     }
-    if ($node->{reftype} eq 'GLOB' and my $glob = $representers->{glob}) {
-        my $code = $glob->{code};
-        return 1 if $code->($self, $node);
-    }
     $node->{data} = $node->{value};
 
 }
